package com.ett.rocketmq.Abstract;

import com.ett.rocketmq.listener.MqMessageEvent;
import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.client.consumer.listener.*;
import org.apache.rocketmq.common.message.MessageExt;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.util.List;

public abstract class AbstractMqConsumer {

    protected DefaultMQPushConsumer consumer;
    // 是否允许顺序消费
    protected boolean isOrderConsumer = false;

    @Autowired
    protected ApplicationEventPublisher publisher;

    /**
     * 初始化consumer，由开发者控制
     *
     * 例如：
     * try {
     *      consumer = new DefaultMQPushConsumer(consumerGroup);
     *      consumer.setNamesrvAddr(namesrvAddr);
     *      consumer.setMessageModel(MessageModel.CLUSTERING);
     *      consumer.setConsumeMessageBatchMaxSize(1);
     *      consumer.setConsumeFromWhere(ConsumeFromWhere.CONSUME_FROM_LAST_OFFSET);
     *      consumer.subscribe("TopicTest", "*");
     *  } catch (MQClientException e) {
     *      e.printStackTrace();
     *  }
     */
    public abstract void start0();

    @PostConstruct
    private void start() {
        if (null == consumer) {
            start0();
        }

        if (isOrderConsumer) {
            consumer.registerMessageListener(new MessageListenerOrderly() {
                @Override
                public ConsumeOrderlyStatus consumeMessage(List<MessageExt> msgs, ConsumeOrderlyContext consumeOrderlyContext) {
                    try {
                        consumeOrderlyContext.setAutoCommit(true);
                        if (null == msgs || msgs.size() == 0) {
                            return ConsumeOrderlyStatus.SUCCESS;
                        }
                        publisher.publishEvent(new MqMessageEvent(consumer, msgs));
                    } catch (Exception e) {
                        e.printStackTrace();
                        return ConsumeOrderlyStatus.SUSPEND_CURRENT_QUEUE_A_MOMENT;
                    }
                    return ConsumeOrderlyStatus.SUCCESS;
                }
            });
        }
        else {
            consumer.registerMessageListener(new MessageListenerConcurrently() {
                @Override
                public ConsumeConcurrentlyStatus consumeMessage(List<MessageExt> msgs, ConsumeConcurrentlyContext consumeConcurrentlyContext) {
                    try {
                        if (null == msgs || msgs.size() == 0) {
                            return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;
                        }
                        publisher.publishEvent(new MqMessageEvent(consumer, msgs));
                    } catch (Exception e) {
                        e.printStackTrace();
                        return ConsumeConcurrentlyStatus.RECONSUME_LATER;
                    }

                    return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;
                }
            });
        }

        new Thread(new Runnable() {
            @Override
            public void run() {

                try {
                    Thread.sleep(5000L);

                    consumer.start();
                    System.err.println("rocketmq consumer server is starting...");
                } catch (InterruptedException e) {
                    e.printStackTrace();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }).start();
    }

    @PreDestroy
    public void shutdown() {
        consumer.shutdown();
    }
}
