package com.ett.rocketmq.consumer;

import com.ett.rocketmq.Abstract.AbstractMqConsumer;
import com.ett.rocketmq.exception.RocketMQException;
import com.ett.rocketmq.listener.MQConsumeMsgListenerProcessor;
import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.common.consumer.ConsumeFromWhere;
import org.apache.rocketmq.common.protocol.heartbeat.MessageModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

@Component
public class MQConsumer extends AbstractMqConsumer {
    public static final Logger LOGGER = LoggerFactory.getLogger(MQConsumer.class);

    @Value("${rocketmq.consumer.groupName}")
    private String groupName;

    @Value("${rocketmq.consumer.namesrvAddr}")
    private String namesrvAddr;

    @Value("${rocketmq.consumer.consumeThreadMin}")
    private int consumeThreadMin;

    @Value("${rocketmq.consumer.consumeThreadMax}")
    private int consumeThreadMax;

    @Value("${rocketmq.consumer.topic}")
    private String topic;

    @Value("${rocketmq.consumer.tag}")
    private String tag;

    @Value("${rocketmq.consumer.consumeMessageBatchMaxSize}")
    private int consumeMessageBatchMaxSize;
    //监听
    @Autowired
    private MQConsumeMsgListenerProcessor mqMessageListenerProcessor;

    @Override
    public void start0() {
        try {
            consumer = new DefaultMQPushConsumer("ett");
            // 设置namesrv地址
            consumer.setNamesrvAddr(namesrvAddr);
            // 设置集群消费
            consumer.setMessageModel(MessageModel.CLUSTERING);

            consumer.setConsumeThreadMin(consumeThreadMin);

            consumer.setConsumeThreadMax(consumeThreadMax);

            // 设置每次消费消息的数量，官方一般建议1条，除非你有批量处理的需求
            consumer.setConsumeMessageBatchMaxSize(consumeMessageBatchMaxSize);
            // 设置消费策略
            consumer.setConsumeFromWhere(ConsumeFromWhere.CONSUME_FROM_LAST_OFFSET);
            // 设置订阅的topic和tags
            consumer.subscribe(topic, tag);
            // ... 根据自己的需求设置consumer其他参数

        } catch (MQClientException e) {
            e.printStackTrace();
        }

    }

    public DefaultMQPushConsumer getRocketMQConsumer() throws RocketMQException {

        DefaultMQPushConsumer consumer = new DefaultMQPushConsumer(groupName);
        consumer.setNamesrvAddr(namesrvAddr);
        consumer.setConsumeThreadMin(consumeThreadMin);
        consumer.setConsumeThreadMax(consumeThreadMax);

        //我们自己实现的监听类
        consumer.registerMessageListener(mqMessageListenerProcessor);
        consumer.setVipChannelEnabled(false);

        /**
         * 设置Consumer第一次启动是从队列头部开始消费还是队列尾部开始消费
         * 如果非第一次启动，那么按照上次消费的位置继续消费
         */
        consumer.setConsumeFromWhere(ConsumeFromWhere.CONSUME_FROM_LAST_OFFSET);
        /**
         * 设置消费模型，集群还是广播，默认为集群
         */
        //consumer.setMessageModel(MessageModel.CLUSTERING);
        /**
         * 设置一次消费消息的条数，默认为1条
         */
        consumer.setConsumeMessageBatchMaxSize(consumeMessageBatchMaxSize);
        try {
          /**
           * 设置该消费者订阅的主题和tag，如果是订阅该主题下的所有tag，则tag使用*；如果需要指定订阅该主题下的某些tag，则使用||分割，例如tag1||tag2||tag3
           */
//            String[] topicTagsArr = topics.split(";");
//            for (String topicTags : topicTagsArr) {
//                String[] topicTag = topicTags.split("~");
//                consumer.subscribe(topicTag[0],topicTag[1]);
//            }
            //订阅
            consumer.subscribe(topic,tag);
            consumer.start();
            LOGGER.info("consumer is start !!! groupName:{},topic:{},namesrvAddr:{}",groupName,topic,namesrvAddr);
        }catch (MQClientException e){
            LOGGER.error("consumer is start !!! groupName:{},topic:{},namesrvAddr:{}",groupName,topic,namesrvAddr,e);
            throw new RocketMQException(e);
        }
        return consumer;
    }
}
