package com.ett.rocketmq.producer;

import com.alibaba.fastjson.JSONObject;
import com.ett.rocketmq.Abstract.AbstractMqProducer;
import com.ett.rocketmq.dto.TestMqMessageDto;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.client.producer.DefaultMQProducer;
import org.apache.rocketmq.client.producer.SendResult;
import org.apache.rocketmq.common.message.Message;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Component
public class MQProducer extends AbstractMqProducer {
    //public static final Logger LOGGER = LoggerFactory.getLogger(MQProducer.class);

    @Value("${rocketmq.producer.instanceName}")
    private String instanceName;

    @Value("${rocketmq.producer.groupName}")
    private String groupName;

    @Value("${rocketmq.producer.namesrvAddr}")
    private String namesrvAddr;
    /**
     * 消息最大大小，默认4M
     */
    @Value("${rocketmq.producer.maxMessageSize}")
    private Integer maxMessageSize ;
    /**
     * 消息发送超时时间，默认3秒
     */
    @Value("${rocketmq.producer.sendMsgTimeout}")
    private Integer sendMsgTimeout;
    /**
     * 消息发送失败重试次数，默认2次
     */
    @Value("${rocketmq.producer.retryTimesWhenSendFailed}")
    private Integer retryTimesWhenSendFailed;

    @Override
    public void start() throws MQClientException {
        if (null == producer) {
            producer = new DefaultMQProducer("ett");
            producer.setNamesrvAddr(namesrvAddr);
            //如果需要同一个jvm中不同的producer往不同的mq集群发送消息，需要设置不同的instanceName
            producer.setInstanceName(instanceName);

            producer.setMaxMessageSize(maxMessageSize);

            producer.setSendMsgTimeout(sendMsgTimeout);
            //如果发送消息失败，设置重试次数，默认为2次
            producer.setRetryTimesWhenSendFailed(retryTimesWhenSendFailed);
        }
        producer.start();
        System.out.println(namesrvAddr);
        System.err.println("rocketmq producer is starting...");
    }

    public boolean send(String topic, String tag, TestMqMessageDto msg) {
        try {
            Message message = new Message(
                    topic, tag,JSONObject.toJSONString(msg).getBytes("utf-8")
            );
            SendResult sendResult = producer.send(message);
            System.err.println("消息生产结果：" + JSONObject.toJSON(sendResult));
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }










//    public DefaultMQProducer getRocketMQProducer() throws RocketMQException {
//        DefaultMQProducer producer;
//        producer = new DefaultMQProducer(groupName
//        );
//        producer.setNamesrvAddr(namesrvAddr);
//        //如果需要同一个jvm中不同的producer往不同的mq集群发送消息，需要设置不同的instanceName
//        producer.setInstanceName(instanceName);
//
//        producer.setMaxMessageSize(maxMessageSize);
//
//        producer.setSendMsgTimeout(sendMsgTimeout);
//        //如果发送消息失败，设置重试次数，默认为2次
//        producer.setRetryTimesWhenSendFailed(retryTimesWhenSendFailed);
//
//        producer.setVipChannelEnabled(false);
//        try {
//            producer.start();
//
//            LOGGER.info(String.format("producer is start ! groupName:[%s],namesrvAddr:[%s]"
//                    , groupName, this.namesrvAddr));
//        } catch (MQClientException e) {
//            LOGGER.error(String.format("producer is error {}"
//                    , e.getMessage(),e));
//            throw new RocketMQException(e);
//        }
//        return producer;
//    }
}
