package com.ett.rocketmq.service;

import com.ett.rocketmq.consumer.MQConsumer;
import com.ett.rocketmq.listener.MqMessageEvent;
import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.client.consumer.listener.MessageListenerConcurrently;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.common.message.MessageExt;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 用于监听MqMessageEvent的服务
 * 消费MQ消息
 *
 * 一般两种方式：
 *     (1)第一种：这个类的作用就是监听SpringEvent事件，
 *     然后再根据消息分发给其他Service进行处理，所以这里一般不会包含业务逻辑代码
 * (2)第二种：这个类的作用就是具体的消费消息类
 */
@Service
public class ConsumerService {

    @Value("${rocketmq.consumer.topic}")
    private String topic;

    @Value("${rocketmq.consumer.tag}")
    private String tag;

//    @Autowired
//    private MqMessageEvent mqMessageEvent;

    /**
     * 消费
     * @param event
     */
    @EventListener(condition = "#event.msgs[0].topic==topic && #event.msgs[0].tags==tag")
    public void testConsumer(MqMessageEvent event) {
       // mqMessageEvent.getConsumer();

        // 由于mq消费者设置了batch=1，所以每次都只会消费一条
        MessageExt msg = event.getMsgs().get(0);
        if (null != msg) {
            // 具体的消费MessageExt的逻辑
            System.out.println(msg);
        }
    }


    /**
     * 消费TopicTestB
     * @param event
     */
//    @EventListener(condition = "#event.msgs[0].topic=='TopicTestB' ")
//    public void testConsumer(MqMessageEvent event) {
//        // 由于mq消费者设置了batch=1，所以每次都只会消费一条
//        MessageExt msg = event.getMsgs().get(0);
//        if (null != msg) {
//            // 具体的消费MessageExt的逻辑
//            if (msg.getTags() == "TagA") {
//                //消费TagA的消息
//            }
//            else if (msg.getTags() == 'TagB'){
//                //消费TagB的消息
//            }
//        }
//    }
}
